searchData={"items":[{"type":"module","title":"Tower","doc":"An automated exception handler for elixir applications.\n\nIt tries to do one job well, **handle** uncaught **error events** in an elixir application\n**and inform** pre-configured list of **reporters** (one or many) about these events.","ref":"Tower.html"},{"type":"module","title":"Reporters - Tower","doc":"You can either:\n  1. use `tower` package directly and [write your own custom reporter](#module-writing-a-custom-reporter) or;\n  1. use one (or many) of the following reporters (separate packages) that build on top and depend on `tower`:\n    * [`TowerEmail`](https://hexdocs.pm/tower_email) ([`tower_email`](https://hex.pm/packages/tower_email))\n    * [`TowerRollbar`](https://hexdocs.pm/tower_rollbar) ([`tower_rollbar`](https://hex.pm/packages/tower_rollbar))\n    * [`TowerSlack`](https://hexdocs.pm/tower_slack) ([`tower_slack`](https://hex.pm/packages/tower_slack))","ref":"Tower.html#module-reporters"},{"type":"module","title":"Enabling automated exception handling - Tower","doc":"Tower.attach()","ref":"Tower.html#module-enabling-automated-exception-handling"},{"type":"module","title":"Disabling automated exception handling - Tower","doc":"Tower.detach()","ref":"Tower.html#module-disabling-automated-exception-handling"},{"type":"module","title":"Manual handling - Tower","doc":"If either, for whatever reason when using automated exception handling, an exception condition is\nnot reaching Tower handling, or you just need or want to manually handle possible errors, you can\nmanually ask Tower to handle exceptions, throws or exits.\n\n    try do\n      # possibly carshing code\n    rescue\n      exception ->\n        Tower.handle_exception(exception, __STACKTRACE__)\n    catch\n      :throw, value ->\n        Tower.handle_throw(value, __STACKTRACE__)\n      :exit, reason ->\n        Tower.handle_exit(reason, __STACKTRACE__)\n    end\n\nor more generally\n\n    try do\n      # possibly carshing code\n    catch\n      kind, reason ->\n        Tower.handle_caught(kind, reason, __STACKTRACE__)\n    end\n\nwhich will in turn call the appropriate function based on the caught `kind` and `reason` values","ref":"Tower.html#module-manual-handling"},{"type":"module","title":"Writing a custom reporter - Tower","doc":"# lib/my_app/error_reporter.ex\n    defmodule MyApp.ErrorReporter do\n      @behaviour Tower.Reporter\n\n      @impl true\n      def report_event(%Tower.Event{} = event) do\n        # do something with event\n\n        # A `Tower.Event` is a struct with the following typespec:\n        #\n        # %Tower.Event{\n        #   id: Uniq.UUID.t(),\n        #   datetime: DateTime.t(),\n        #   level: :logger.level(),\n        #   kind: :error | :exit | :throw | :message,\n        #   reason: Exception.t() | term(),\n        #   stacktrace: Exception.stacktrace() | nil,\n        #   log_event: :logger.log_event() | nil,\n        #   plug_conn: struct() | nil,\n        #   metadata: map()\n        # }\n      end\n    end\n\n    # in some config/*.exs\n    config :tower, reporters: [MyApp.ErrorReporter]\n\n    # config/application.ex\n    Tower.attach()\n\n`Tower.attach/0` will be responsible for registering the necessary handlers in your application\nso that any uncaught exception, uncaught throw or abnormal process exit is handled by Tower and\npassed along to the reporter.","ref":"Tower.html#module-writing-a-custom-reporter"},{"type":"function","title":"Tower.attach/0","doc":"Attaches the necessary handlers to automatically listen for application errors.\n\n[Adds](https://www.erlang.org/doc/apps/kernel/logger.html#add_handler/3) a new\n[`logger_handler`](https://www.erlang.org/doc/apps/kernel/logger_handler), which listens for all\nuncaught exceptions, uncaught throws, abnormal process exits, among other log events of interest.\n\nAdditionally adds other handlers specifically tailored for some packages that\ndo catch errors and have their own specific error handling and emit events instead\nof letting errors get to the logger handler, like oban or bandit.\n\nNote that `Tower.attach/0` is not a precondition for `Tower` `handle_*` functions to work\nproperly and inform reporters. They are independent.","ref":"Tower.html#attach/0"},{"type":"function","title":"Tower.detach/0","doc":"Detaches the handlers.\n\nThat means it stops the automatic handling of errors.\nYou can still manually call `Tower` `handle_*` functions and reporters will be informed about\nthose events.","ref":"Tower.html#detach/0"},{"type":"function","title":"Tower.equal_or_greater_level?/2","doc":"Compares event level severity.\n\nReturns true if `level1` severity is equal or greater than `level2` severity.","ref":"Tower.html#equal_or_greater_level?/2"},{"type":"function","title":"Examples - Tower.equal_or_greater_level?/2","doc":"iex> Tower.equal_or_greater_level?(:emergency, :error)\n    true\n    iex> Tower.equal_or_greater_level?(%Tower.Event{level: :info}, :info)\n    true\n    iex> Tower.equal_or_greater_level?(:warning, :critical)\n    false","ref":"Tower.html#equal_or_greater_level?/2-examples"},{"type":"function","title":"Tower.handle_caught/4","doc":"Asks Tower to handle a manually caught error.","ref":"Tower.html#handle_caught/4"},{"type":"function","title":"Example - Tower.handle_caught/4","doc":"try do\n      # possibly crashing code\n    catch\n      # Note this will also catch and handle normal (`:normal` and `:shutdown`) exits\n      kind, reason ->\n        Tower.handle_caught(kind, reason, __STACKTRACE__)\n    end","ref":"Tower.html#handle_caught/4-example"},{"type":"function","title":"Options - Tower.handle_caught/4","doc":"* `:plug_conn` - a `Plug.Conn` relevant to the event, if available, that may be used\n  by reporters to report useful context information. Be aware that the `Plug.Conn` may contain\n  user and/or system sensitive information, and it's up to each reporter to be cautious about\n  what to report or not.\n  * `:metadata` - a `Map` with additional information you want to provide about the event. It's\n  up to each reporter if and how to handle it.","ref":"Tower.html#handle_caught/4-options"},{"type":"function","title":"Tower.handle_exception/3","doc":"Asks Tower to handle a manually caught exception.","ref":"Tower.html#handle_exception/3"},{"type":"function","title":"Example - Tower.handle_exception/3","doc":"try do\n      # possibly crashing code\n    rescue\n      exception ->\n        Tower.handle_exception(exception, __STACKTRACE__)\n    end","ref":"Tower.html#handle_exception/3-example"},{"type":"function","title":"Options - Tower.handle_exception/3","doc":"* Accepts same options as `handle_caught/4#options`.","ref":"Tower.html#handle_exception/3-options"},{"type":"function","title":"Tower.handle_exit/3","doc":"Asks Tower to handle a manually caught exit.","ref":"Tower.html#handle_exit/3"},{"type":"function","title":"Example - Tower.handle_exit/3","doc":"try do\n      # possibly exiting code\n    catch\n      # Note this will also catch and handle normal (`:normal` and `:shutdown`) exits\n      :exit, exit_reason ->\n        Tower.handle_exit(exit_reason, __STACKTRACE__)\n    end","ref":"Tower.html#handle_exit/3-example"},{"type":"function","title":"Options - Tower.handle_exit/3","doc":"* Accepts same options as `handle_caught/4#options`.","ref":"Tower.html#handle_exit/3-options"},{"type":"function","title":"Tower.handle_message/3","doc":"Asks Tower to handle a message of certain level.","ref":"Tower.html#handle_message/3"},{"type":"function","title":"Examples - Tower.handle_message/3","doc":"Tower.handle_message(:emergency, \"System is falling apart\")\n\n    Tower.handle_message(:error, \"Unknown error has ocurred\", metadata: %{any_key: \"here\"})\n\n    Tower.handle_message(:info, \"Just something interesting\", metadata: %{interesting: \"additional data\"})","ref":"Tower.html#handle_message/3-examples"},{"type":"function","title":"Options - Tower.handle_message/3","doc":"* Accepts same options as `handle_caught/4#options`.","ref":"Tower.html#handle_message/3-options"},{"type":"function","title":"Tower.handle_throw/3","doc":"Asks Tower to handle a manually caught throw.","ref":"Tower.html#handle_throw/3"},{"type":"function","title":"Example - Tower.handle_throw/3","doc":"try do\n      # possibly throwing code\n    catch\n      thrown_value ->\n        Tower.handle_throw(thrown_value, __STACKTRACE__)\n    end","ref":"Tower.html#handle_throw/3-example"},{"type":"function","title":"Options - Tower.handle_throw/3","doc":"* Accepts same options as `handle_caught/4#options`.","ref":"Tower.html#handle_throw/3-options"},{"type":"module","title":"Tower.EphemeralReporter","doc":"A very slim and naive built-in reporter, that just stores Tower events as process state.\n\nPossibly useful for development or testing.","ref":"Tower.EphemeralReporter.html"},{"type":"module","title":"Example - Tower.EphemeralReporter","doc":"iex> {:ok, pid} = Tower.EphemeralReporter.start_link([])\n    iex> Tower.EphemeralReporter.events()\n    []\n    iex> Application.put_env(:tower, :reporters, [Tower.EphemeralReporter])\n    iex> Tower.attach()\n    :ok\n    iex> spawn(fn -> 1 / 0 end)\n    iex> Process.sleep(200)\n    :ok\n    iex> [event] = Tower.EphemeralReporter.events()\n    iex> event.kind\n    :error\n    iex> event.reason\n    %ArithmeticError{message: \"bad argument in arithmetic expression\"}\n    iex> Tower.detach()\n    :ok\n    iex> Tower.EphemeralReporter.stop(pid)\n    :ok","ref":"Tower.EphemeralReporter.html#module-example"},{"type":"function","title":"Tower.EphemeralReporter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Tower.EphemeralReporter.html#child_spec/1"},{"type":"function","title":"Tower.EphemeralReporter.events/0","doc":"Returns the list of all stored events.","ref":"Tower.EphemeralReporter.html#events/0"},{"type":"function","title":"Tower.EphemeralReporter.start_link/1","doc":"","ref":"Tower.EphemeralReporter.html#start_link/1"},{"type":"function","title":"Tower.EphemeralReporter.stop/1","doc":"","ref":"Tower.EphemeralReporter.html#stop/1"},{"type":"module","title":"Tower.Event","doc":"A struct representing a captured event.\n\nTower converts every captured error and message into a struct of this type\nbefore passing along to reporters.","ref":"Tower.Event.html"},{"type":"type","title":"Tower.Event.error_kind/0","doc":"","ref":"Tower.Event.html#t:error_kind/0"},{"type":"type","title":"Tower.Event.level/0","doc":"","ref":"Tower.Event.html#t:level/0"},{"type":"type","title":"Tower.Event.non_error_kind/0","doc":"","ref":"Tower.Event.html#t:non_error_kind/0"},{"type":"type","title":"Tower.Event.reason/0","doc":"","ref":"Tower.Event.html#t:reason/0"},{"type":"type","title":"Tower.Event.t/0","doc":"A struct representing a captured event.\n\nTower converts every captured error and message into a struct of this type\nbefore passing along to reporters.","ref":"Tower.Event.html#t:t/0"},{"type":"behaviour","title":"Tower.Reporter","doc":"Behaviour that can be implemented to write Tower reporters.\n\nTower comes built-in with a very naive `Tower.EphemeralReporter`, that implements\nthis behavior, which can be useful for development and testing purposes.\n\nSeparate packages that implement this behaviour:\n\n  * [tower_email](https://hex.pm/packages/tower_email) ([`TowerEmail`](https://hexdocs.pm/tower_email))\n  * [tower_rollbar](https://hex.pm/packages/tower_rollbar) ([`TowerRollbar`](https://hexdocs.pm/tower_rollbar))\n  * [tower_slack](https://hex.pm/packages/tower_slack) ([`TowerSlack`](https://hexdocs.pm/tower_slack))","ref":"Tower.Reporter.html"},{"type":"callback","title":"Tower.Reporter.report_event/1","doc":"Function that will be called with every event handled by Tower.","ref":"Tower.Reporter.html#c:report_event/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.5.0] - 2024-08-20","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Oban support\n  - I.e: Automatic handling of errors occurring inside an Oban job perform.\n- Bandit support\n  - I.e: Automatic handling of errors occurring inside plug dispatch call when using `Bandit` adapter.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Properly handle exits ocurring inside a plug dispatch call when using `Plug.Cowboy` adapter.\n\n## [0.4.0] - 2024-08-16","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `Tower.Event` `plug_conn` field with a `Plug.Conn` if available during handling.","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Drop support for elixir 1.13 and 1.14 which were not yet fully integrated with `:logger` handlers, which\nis the main way `tower` handles errors. Keep elixir 1.15+ for now.\n\n## [0.3.0] - 2024-08-15","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- `Tower.Event` `datetime` field with a `DateTime` value with microseconds precision.","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- `Tower.Event` `time` field.","ref":"changelog.html#removed"},{"type":"extras","title":"Changed - Changelog","doc":"- `Tower.Event` `time` field replaced with `datetime` field.\n\n## [0.2.0] - 2024-08-09","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New `Tower.Event` struct to represent and encapsulate any exception, exit, throw or message.\n- New `time` field of `Tower.Event` struct, including the timestamp with microseconds precision.\n- New `id` field of `Tower.Event` struct, including a timed-based sortable unique value (UUIDv7).\n- Ablility to manually handle/report exceptions, exits and throws\n  - `Tower.handle_exception/2,3`\n  - `Tower.handle_exit/2,3`\n  - `Tower.handle_throw/2,3`\n- `metadata` field supporting user reported metadata\n- `Tower.handle_caught/3,4` for easier handling of `catch kind, reason`\n- `Tower.equal_or_greater_level?/2` to aid reporters in comparing log level, e.g. supporting per-reporter reporting level.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Reporters (those implementing `Tower.Reporter` behaviour) can now handle events with just one callback: `report_event/1`, in replacement of `report_exception/2,3`, `report_exit/2,3`, `report_throw/2,3` and `report_message/2,3` callbacks.\n\n[0.5.0]: https://github.com/mimiquate/tower/compare/v0.4.0...v0.5.0/\n[0.4.0]: https://github.com/mimiquate/tower/compare/v0.3.0...v0.4.0/\n[0.3.0]: https://github.com/mimiquate/tower/compare/v0.2.0...v0.3.0/\n[0.2.0]: https://github.com/mimiquate/tower/compare/v0.1.0...v0.2.0/","ref":"changelog.html#changed"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}